package com.example.darktooth;

import android.app.Activity;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.Process;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.UUID;

public class BluetoothForegroundService extends Service {

    private Looper scanLooper;
    private ScanHandler scanHandler;
    BluetoothAdapter bluetoothAdapter;
    BroadcastReceiver scanReceiver;
    IntentFilter filter;
    final int hasAddress;
    BluetoothServerSocket bluetoothServer;

    public BluetoothForegroundService() {
        hasAddress = 0;
    }

    private final class ScanHandler extends Handler {

        public ScanHandler(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            // thread for handling periodic bluetooth scans
            while (true) {
                System.out.println("## SCANNING..!");
                bluetoothAdapter.startDiscovery();
                try {
                    Thread.sleep(12000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                bluetoothAdapter.cancelDiscovery();
                try {
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private final class ServeHandler extends Handler {

        public ServeHandler(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            // thread for handling lookup requests from other devices.
            while (true) {
                BluetoothSocket socket = null;
                try {
                    socket = bluetoothServer.accept();
                } catch(IOException e) {
                    Log.d("serveHandler","Could not connect: " + e.toString());
                    try {
                        bluetoothServer.close();
                    } catch(IOException close) {
                        Log.d("serveHandler", "Could not close connection:" + e.toString());
                    }
                }
                if (socket != null) {
                    try {
                        final OutputStream inStream = socket.getOutputStream();
                        BluetoothDevice device = socket.getRemoteDevice();
                        inStream.write(device.getAddress().getBytes());
                    } catch (IOException e) {
                        Log.d("serveHandler", "Could not get InputStream:" + e.toString());
                    }
                    try {
                        socket.close();
                    } catch (IOException e) {
                        Log.d("serveHandler", "Could not close connection:" + e.toString());
                    }
                }

            }
        }
    }

    @Override
    public void onCreate() {
        System.out.println("## CREAING SCANNING SERVICE!");
        HandlerThread thread = new HandlerThread("ServiceStartArguments",
                Process.THREAD_PRIORITY_BACKGROUND);
        thread.start();
        scanLooper = thread.getLooper();
        scanHandler = new ScanHandler(scanLooper);
        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        scanReceiver = new BroadcastReceiver() {
            public void onReceive(Context context, Intent intent) {
                String action = intent.getAction();
                if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                    BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    System.out.println("a device is found:");
                    System.out.println(device.getAddress());

                    if (hasAddress == 0) {
                        // here we should establish connection to the peer to get own address
                        System.out.println("we don't have an address, trying to connect to get!");
                        if (Arrays.asList(device.getUuids()).contains(UUID.fromString("9f456c30-795f-11ea-bc55-0242ac130003"))) {
                            try {
                                BluetoothSocket socket =
                                        device.createRfcommSocketToServiceRecord(UUID.fromString("9f456c30-795f-11ea-bc55-0242ac130003"))
                                socket.connect();
                                try {
                                    byte []myAddress = new byte[128];
                                    final InputStream outStream = socket.getInputStream();
                                    outStream.read(myAddress);
                                } catch (IOException e) {
                                    Log.d("serveHandler", "Could not get InputStream:" + e.toString());
                                }
                                try {
                                    socket.close();
                                } catch (IOException e) {
                                    Log.d("serveHandler", "Could not close connection:" + e.toString());
                                }
                            } catch (IOException e) {
                                e.printStackTrace();
                            };
                        }

                    } else {
                        // here we should just store the addresses
                    }
                }
            }
        };
        filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(scanReceiver, filter);
        try {
            bluetoothServer = bluetoothAdapter.listenUsingRfcommWithServiceRecord("1337",
                    UUID.fromString("9f456c30-795f-11ea-bc55-0242ac130003"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        System.out.println("## STARTING SCANNING THREAD!");
        Message msg = scanHandler.obtainMessage();
        scanHandler.sendMessage(msg);
        return START_STICKY;
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onDestroy() {
        this.registerReceiver(scanReceiver, filter);
    }
}