/*
  Handling needed permissions:
    - bluetooth: should be always enabled. [x]
    - run on boot. [-]
    - ignore battery optimization. [x]
    - gps. [x]
*/

import 'package:permission_handler/permission_handler.dart';

class PermissionService {
  final PermissionHandler _permissionHandler = PermissionHandler();
  Future<bool> _requestPermission(PermissionGroup permission) async {
    var result = await _permissionHandler.requestPermissions([permission]);
    if (result[permission] == PermissionStatus.granted) {
      return true;
    }
    return false;
  }

  Future<bool> hasPermission(PermissionGroup permission) async {
    var permissionStatus =
        await _permissionHandler.checkPermissionStatus(permission);
    return permissionStatus == PermissionStatus.granted;
  }

  Future<bool> requestBluetoothPermission() async {
    return _requestPermission(PermissionGroup.location);
  }

  Future<bool> requestBatteryOptPermission() async {
    return _requestPermission(PermissionGroup.ignoreBatteryOptimizations);
  }
}